/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */
package com.javacodegeeks.animal;

import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.annotation.JsonTypeName;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.javacodegeeks.animal.Animal.Cat;
import com.javacodegeeks.animal.Animal.Dog;
import java.io.IOException;

/**
 *
 * @author omozegieaziegbe
 */
@JsonTypeInfo(
        use = JsonTypeInfo.Id.NAME,
        include = JsonTypeInfo.As.PROPERTY,
        property = "type"
)
@JsonSubTypes({
    @JsonSubTypes.Type(value = Dog.class, name = "dog"),
    @JsonSubTypes.Type(value = Cat.class, name = "cat")
})
public class Animal {

    public Animal() {
    }

    private String name;

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @JsonTypeName("dog")
    public static class Dog extends Animal {

        public Dog() {
        }

        // Dog-specific fields and methods
        private String breed;

        public String getBreed() {
            return breed;
        }

        public void setBreed(String breed) {
            this.breed = breed;
        }

    }

    @JsonTypeName("cat")
    public static class Cat extends Animal {

        // Cat-specific fields and methods
        private int numberOfLives;

        public Cat() {
        }

        public int getNumberOfLives() {
            return numberOfLives;
        }

        public void setNumberOfLives(int numberOfLives) {
            this.numberOfLives = numberOfLives;
        }

    }

    public static void main(String[] args) throws IOException {

        //Create an instance of the ObjectMapper class and configure it to handle polymorphic deserialization.
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.enableDefaultTypingAsProperty(ObjectMapper.DefaultTyping.NON_FINAL, "type");
        
        //Deserialize your JSON data into the base class
        String json = "{\"type\":\"dog\",\"name\":\"Fido\",\"breed\":\"Labrador\"}";
        Animal animal = objectMapper.readValue(json, Animal.class);
        
        String jsonOutput = objectMapper.writeValueAsString(animal);
        
        System.out.println("" + jsonOutput);

    }
}
