/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */
package com.javacodegeeks.shape;

import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.javacodegeeks.shape.Shape.Circle;
import com.javacodegeeks.shape.Shape.Rectangle;
import java.io.IOException;
import java.util.Map;
import java.util.Set;
import org.reflections.Reflections;

/**
 *
 * @author omozegieaziegbe
 */
// Base class or interface
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY, property = "type")
@JsonSubTypes({
    @JsonSubTypes.Type(value = Circle.class, name = "circle"),
    @JsonSubTypes.Type(value = Rectangle.class, name = "rectangle")
})
public class Shape {

    public Shape() {
    }
    
    // Common properties and methods

    public static class Circle extends Shape {

        public Circle() {
        }

        private double radius;
        
        
        // Circle-specific properties and methods

        public double getRadius() {
            return radius;
        }

        public void setRadius(double radius) {
            this.radius = radius;
        }
    }

    public static class Rectangle extends Shape {

        public Rectangle() {
        }

        private double width;
        private double height;
        // Rectangle-specific properties and methods

        public double getWidth() {
            return width;
        }

        public void setWidth(double width) {
            this.width = width;
        }

        public double getHeight() {
            return height;
        }

        public void setHeight(double height) {
            this.height = height;
        }
        

    }

    public static void main(String[] args) throws IOException {
        
        
        ObjectMapper objectMapper = new ObjectMapper();
        String json = "{\"type\":\"circle\",\"radius\":5.0}";

// Deserialize the JSON into a Map to extract the type information
        Map<String, Object> jsonMap = objectMapper.readValue(json, new TypeReference<Map<String, Object>>() {
        });
        String type = (String) jsonMap.get("type");

// Use Reflections to find the corresponding Java class
        Reflections reflections = new Reflections("com.javacodegeeks.shape");
        Set<Class<? extends Shape>> subTypes = reflections.getSubTypesOf(Shape.class);

        Class<? extends Shape> shapeClass = subTypes.stream()
                .filter(clazz -> clazz.getSimpleName().equalsIgnoreCase(type))
                .findFirst()
                .orElseThrow(() -> new IllegalArgumentException("Unknown type: " + type));

// Deserialize the JSON into the determined class
        Shape shape = objectMapper.convertValue(jsonMap, shapeClass);

        String jsonOutput = objectMapper.writeValueAsString(shape);

        System.out.println("" + jsonOutput);
    }
}
